# imports - compatibility imports
from pipupgrade._compat import StringIO

# imports - test imports
import pytest

# imports - test imports
from testutils import mock_input

def test_command():
    pass